<?php
/**
 * The MIT License
 *
 * Copyright (c) 2007 Andy Smith
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

namespace LTI;

/**
 * A Trivial memory-based store - no support for tokens
 */
class TrivialOAuthDataStore extends OAuthDataStore {
	private $consumers = array();

	function add_consumer($consumer_key, $consumer_secret) {
		$this->consumers[$consumer_key] = $consumer_secret;
	}

	function lookup_consumer($consumer_key) {
		if (strpos($consumer_key, "http://" ) === 0) {
			$consumer = new OAuthConsumer($consumer_key,"secret", NULL);
			return $consumer;
		}
		if ($this->consumers[$consumer_key]) {
			$consumer = new OAuthConsumer($consumer_key,$this->consumers[$consumer_key], NULL);
			return $consumer;
		}
		return NULL;
	}

	function lookup_token($consumer, $token_type, $token) {
		return new OAuthToken($consumer, "");
	}

	// Return NULL if the nonce has not been used
	// Return $nonce if the nonce was previously used
	function lookup_nonce($consumer, $token, $nonce, $timestamp) {
		// Should add some clever logic to keep nonces from
		// being reused - for no we are really trusting
		// that the timestamp will save us
		return NULL;
	}

	function new_request_token($consumer) {
			return NULL;
	}

	function new_access_token($token, $consumer) {
			return NULL;
	}
}
