(function () {
    'use strict';

    var currentMee;
    var getCurrentMee = function () {
      if (currentMee)
        return currentMee;
      return null;
    };
    var getFrameForDocument = function (document) {
      var w = document.defaultView || document.parentWindow;
      var frames = w.parent.document.getElementsByTagName('iframe');
      for (var i = frames.length; i-- > 0;) {
        var frame = frames[i];
        try {
          var d = frame.contentDocument || frame.contentWindow.document;
          if (d === document)
            return frame;
        } catch (e) {
        }
      }
    };
    var updateMEE = function (frame, w, h) {
      var iframe = $(getFrameForDocument(frame.document));
      iframe.css('width', w + 5 + 'px');
      iframe.css('height', h + 'px');
      iframe.css('vertical-align', 'middle');
    };
    var clickMEEiFrame = function (frame) {
      if (frame.document)
        frame = getFrameForDocument(frame.document);
      else
        frame = $(frame).children()[0];
      var e = tinymce.activeEditor;
      e.selection.select(frame);
      e.execCommand('mceMEE');
    };
    var setup = function (editor, url) {
      editor.addCommand('mceMEE', function () {
        editor.selection.select(editor.plugins['maths-equation-editor'].getCurrentMee());
        tinymce.activeEditor.windowManager.open({
          title: 'mee',
          body: {
            type: 'panel',
            items: [{
                type: 'htmlpanel',
                html: '<iframe id="meedialog" name="meedialog" src="' + url + '/dialog.html" frameborder="0"></iframe>'
              }]
          },
          size: 'large',
          buttons: [
            {
              type: 'custom',
              text: 'insert',
              name: 'insert',
              disabled: currentMee ? true : false
            },
            {
              type: 'custom',
              text: 'update',
              name: 'update',
              disabled: currentMee ? false : true
            },
            {
              type: 'cancel',
              text: 'cancel',
              name: 'cancel',
              primary: true
            }
          ],
          onAction: function (dialogApi) {
            window['meedialog'].insertMME();
            dialogApi.close();
          }
        });
        $('.tox-form__group').css('height', '100%');
        $('#meedialog').css('height', '100%');
        $('#meedialog').css('width', '100%');
      });
      editor.ui.registry.addButton('maths-equation-editor', {
        icon: 'mee',
        tooltip: 'mee',
        onAction: function () {
          tinymce.activeEditor.windowManager.open({
            title: 'mee',
            body: {
              type: 'panel',
              items: [{
                  type: 'htmlpanel',
                  html: '<iframe id="meedialog" name="meedialog" src="' + url + '/dialog.html" frameborder="0" width="100%" height="100%"></iframe>'
                }]
            },
            size: 'large',
            buttons: [
              {
                type: 'custom',
                text: 'insert',
                name: 'insert',
                disabled: currentMee ? true : false
              },
              {
                type: 'custom',
                text: 'update',
                name: 'update',
                disabled: currentMee ? false : true
              },
              {
                type: 'cancel',
                text: 'cancel',
                name: 'cancel',
                primary: true
              }
            ],
            onAction: function (dialogApi) {
              window['meedialog'].insertMME();
              dialogApi.close();
            }
          });
          $('.tox-form__group').css('height', '100%');
          $('#meedialog').css('height', '100%');
          $('#meedialog').css('width', '100%');
        },
        onSetup: function (buttonApi) {
          var editorEventCallback = function (eventApi) {
            buttonApi.setDisabled(eventApi.element.nodeName.toLowerCase() === 'maths-equation-editor');
            var mee = findMee(eventApi.element.childNodes[0]);
            if (mee) {
              currentMee = mee;
              $(mee).css('border', '1px solid blue');
            } else {
              currentMee = null;
            }
          };
          editor.on('NodeChange', editorEventCallback);
          return function (buttonApi) {
            editor.off('NodeChange', editorEventCallback);
          };
        }
      });
      editor.on('SetContent', function (evt) {
        var body = editor.getBody();
        var elems = $(body).find('.mee');
        for (var i = 0; i < elems.length; i++) {
          var elem = elems[i];
          var data = {};
          var eltype = elem.tagName;
          if (eltype == 'DIV') {
            data.inline = false;
          } else {
            data.inline = true;
          }
          data.latex = encodeQuotes($(elem).html());
          data.fontsize = $(elem).css('font-size');
          var datatxt = JSON.stringify(data);
          var html = '<iframe class=\'mee_iframe\' src=\'' + url + '/frame.html?' + datatxt + '\' frameborder=\'0\'></iframe>';
          var newelem = $(html);
          $(newelem).insertBefore(elem);
          if (!data.inline)
            newelem.attr('align', 'middle');
          $(elem).remove();
        }
      });
      editor.on('SaveContent', function (evt) {
        var doc = $('<div>');
        doc.html(evt.content);
        $(doc).find('.mee_iframe').each(function () {
          var src = $(this).attr('src');
          var data = src.substr(src.indexOf('?'));
          data = data.substr(1);
          data = $.parseJSON(data);
          data.latex = unencodeQuotes(data.latex);
          $(this).removeClass('mee_iframe');
          var newelem;
          if (data.inline) {
            newelem = $('<span>');
          } else {
            newelem = $('<div>');
          }
          $(newelem).html(data.latex);
          $(newelem).addClass('mee');
          $(newelem).insertBefore(this);
          $(this).remove();
        });
        evt.content = doc.html();
      });
      function unencodeQuotes(str) {
        str = str.replace(/~quot~/g, '\'');
        str = str.replace(/~dblquot~/g, '"');
        return str;
      }
      function encodeQuotes(str) {
        str = str.replace(/'/g, '~quot~');
        str = str.replace(/"/g, '~dblquot~');
        return str;
      }
      function findMee(element) {
        if ($(element).hasClass('mee_iframe'))
          return element;
        if (!element.parentNode)
          return null;
        return findMee(element.parentNode);
      }
      return get$1();
    };
    var get$1 = function (editor) {
      return {
        getCurrentMee: function () {
          return getCurrentMee();
        },
        updateMEE: function (frame, w, h) {
          return updateMEE(frame, w, h);
        },
        clickMEEiFrame: function (frame) {
          return clickMEEiFrame(frame);
        }
      };
    };
    function Plugin () {
      var supportedLangs = [
        'en',
        'cs',
        'pl',
        'sk'
      ];
      supportedLangs.forEach(function (item) {
        tinymce.PluginManager.requireLangPack('maths-equation-editor', item);
      });
      tinymce.PluginManager.add('maths-equation-editor', setup);
    }

    Plugin();

})();
