(function () {
    'use strict';

    var setup = function (editor, url) {
      editor.ui.registry.addButton('ruby-annotation', {
        icon: 'ruby',
        tooltip: 'ruby',
        onAction: function () {
          tinymce.activeEditor.windowManager.open({
            title: 'ruby',
            body: {
              type: 'panel',
              items: [
                {
                  type: 'textarea',
                  name: 'rb',
                  label: 'text',
                  placeholder: 'text'
                },
                {
                  type: 'htmlpanel',
                  html: '<style>#rubyPreview {text-align: center;font-size: 2em;}.wrap > ruby > rt {font-size: 0.5em;}</style><div class="previewWrapper"><h3>preview</h3><div id="rubyPreview"></div></div>'
                }
              ]
            },
            buttons: [
              {
                type: 'submit',
                text: 'add',
                name: 'add',
                disabled: true
              },
              {
                type: 'cancel',
                text: 'cancel',
                name: 'cancel',
                primary: true
              }
            ],
            onChange: function (dialogApi, details) {
              var data = dialogApi.getData();
              var toggle = data.rb ? dialogApi.enable : dialogApi.disable;
              toggle('add');
              document.getElementById('rubyPreview').innerHTML = constructTag(data.rb);
            },
            onSubmit: function (dialogApi) {
              var data = dialogApi.getData();
              tinymce.activeEditor.execCommand('mceInsertContent', false, constructTag(data.rb));
              dialogApi.close();
            },
            initialData: { rb: init() }
          });
        }
      });
    };
    function Plugin () {
      var supportedLangs = [
        'en',
        'cs',
        'pl',
        'sk'
      ];
      supportedLangs.forEach(function (item) {
        tinymce.PluginManager.requireLangPack('ruby-annotation', item);
      });
      tinymce.PluginManager.add('ruby-annotation', setup);
    }function constructTag(rbValue) {
      var ruby = '';
      var k = 0;
      for (var l = 0; l < rbValue.length; l++) {
        if (rbValue.charAt(l) == '(') {
          l++;
          k++;
          ruby += '<rp>{</rp><rt>' + rbValue.charAt(l);
        } else if (rbValue.charAt(l) == ')') {
          ruby += '</rt><rp>}</rp></ruby>';
          k--;
        } else if (k > 0) {
          ruby += rbValue.charAt(l);
        } else {
          ruby += '<ruby><rb>' + rbValue.charAt(l) + '</rb>';
          if (rbValue.charAt(l + 1) != '(') {
            ruby += '</ruby>';
          }
        }
      }
      return '<span class=\'wrap\'>' + ruby + '</span>';
    }
    function init() {
      var value = '';
      var rubyValue = tinymce.DOM.getParent(tinymce.activeEditor.selection.getNode(), 'span');
      if (rubyValue !== null) {
        var elements = rubyValue.getElementsByTagName('ruby');
        for (var i = 0; i < elements.length; i++) {
          if (elements[i].textContent.indexOf('{') > -1) {
            var rt = elements[i].getElementsByTagName('rt')[0].innerHTML;
            value += elements[i].textContent.split('{')[0] + '(' + rt + ')';
          } else {
            value += elements[i].textContent;
          }
        }
      }
      return value;
    }

    Plugin();

})();
